Package: SPOTlight
Version: 1.17.0
Type: Package
Title: `SPOTlight`: Spatial Transcriptomics Deconvolution
Description: `SPOTlight` provides a method to deconvolute spatial
        transcriptomics spots using a seeded NMF approach along with
        visualization tools to assess the results. Spatially resolved
        gene expression profiles are key to understand tissue
        organization and function. However, novel spatial
        transcriptomics (ST) profiling techniques lack single-cell
        resolution and require a combination with single-cell RNA
        sequencing (scRNA-seq) information to deconvolute the spatially
        indexed datasets. Leveraging the strengths of both data types,
        we developed SPOTlight, a computational tool that enables the
        integration of ST with scRNA-seq data to infer the location of
        cell types and states within a complex tissue. SPOTlight is
        centered around a seeded non-negative matrix factorization
        (NMF) regression, initialized using cell-type marker genes and
        non-negative least squares (NNLS) to subsequently deconvolute
        ST capture locations (spots).
Authors@R: c(
  person("Marc", "Elosua-Bayes", email="elosua.marc@gmail.com", role=c("aut", "cre")),
  person("Zachary", "DeBruine", email="zacharydebruine@gmail.com", role="aut"),
  person("Helena L.", "Crowell", email="helena@crowell.eu", role="aut"))
Depends: R (>= 4.5.0)
Imports: ggplot2, Matrix, SingleCellExperiment, sparseMatrixStats,
        stats
Suggests: BiocStyle, colorBlindness, DelayedArray, DropletUtils,
        ExperimentHub, ggcorrplot, grDevices, grid, igraph, jpeg,
        knitr, methods, png, rmarkdown, scater, scatterpie, scran,
        SpatialExperiment, SummarizedExperiment, S4Vectors,
        TabulaMurisSenisData, TENxVisiumData, testthat
LinkingTo: Rcpp, RcppEigen
biocViews: SingleCell, Spatial, StatisticalMethod
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: knitr
URL: https://github.com/MarcElosua/SPOTlight
BugReports: https://github.com/MarcElosua/SPOTlight/issues
Config/pak/sysreqs: zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:58:11 UTC
RemoteUrl: https://github.com/bioc/SPOTlight
RemoteRef: HEAD
RemoteSha: ad59a485ef80aea3679fb1839aa622723b016ec4
NeedsCompilation: yes
Packaged: 2026-05-30 09:48:52 UTC; root
Author: Marc Elosua-Bayes [aut, cre],
  Zachary DeBruine [aut],
  Helena L. Crowell [aut]
Maintainer: Marc Elosua-Bayes <elosua.marc@gmail.com>
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-30 10:17:10 UTC; unix
