# Generated by roxygen2: do not edit by hand

export(SPOTlight)
export(getMGS)
export(mockSC)
export(mockSP)
export(plotCorrelationMatrix)
export(plotImage)
export(plotInteractions)
export(plotSpatialScatterpie)
export(plotTopicProfiles)
export(runDeconvolution)
export(trainNMF)
import(ggplot2)
importFrom(Matrix,Matrix)
importFrom(Matrix,colSums)
importFrom(Matrix,rowSums)
importFrom(Matrix,t)
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SingleCellExperiment,cbind)
importFrom(SingleCellExperiment,colLabels)
importFrom(SingleCellExperiment,counts)
importFrom(grid,rasterGrob)
importFrom(grid,unit)
importFrom(sparseMatrixStats,colMedians)
importFrom(sparseMatrixStats,rowAlls)
importFrom(sparseMatrixStats,rowSds)
importFrom(sparseMatrixStats,rowSums2)
importFrom(stats,aggregate)
importFrom(stats,cor)
importFrom(stats,median)
importFrom(stats,rnbinom)
importFrom(stats,runif)
useDynLib(SPOTlight, .registration = TRUE)
